/***************************************************************************
                          qtabman.h  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id$
 *
 */

/* QT includes */
#ifndef QTABMAN
#define QTABMAN
#define NO_DEBUG
#include <qwindowsstyle.h>
#include <qmotifstyle.h>
#include <qcdestyle.h>
#include <qplatinumstyle.h>
#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qevent.h>
#include <qsplitter.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qmultilinedit.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qkeycode.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qcursor.h> 
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qvaluelist.h>
#include <qgroupbox.h>
#include <qtooltip.h>

/* other includes */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <pthread.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>

/* pixmaps */
#include "column.xpm"
#include "constraint.xpm"
#include "index.xpm"
#include "package.xpm"
#include "packagebody.xpm"
#include "procedure.xpm"
#include "table.xpm"
#include "trigger.xpm"
#include "view.xpm"
#include "exit.xpm"
#include "function.xpm"


/* prototypes */
#include "prototypes.h"

/* defines */
#define STR_TO_VARCHAR(str,vc) { strcpy(vc.arr,str); vc.len=strlen(str); }
#define VARCHAR_TO_STR(vc,str) { vc.arr[vc.len]=0; strcpy(str,vc.arr0; }
#define NT_VARCHAR(vc) { vc.arr[vc.len]=0; }
#define VERSION "Qtabman: 1.0.0 (Development Release 2)."

/* Class defintions */
class SchemaTree;
class SchemaTreeItem;
class SchemaList;
class SchemaListItem;
class LoginDialog;
class DebugDialog;
class TabmanDialog;
class TableDialog;
class ModelDialog;
class LoginDialog;
class ApplicationWinow;
class ScrollText;
class AboutDialog;
class StatusBar;
class MenuBar;
class ToolBar;


/********************************************************************************
*********************************************************************************
* Dialog classes
*********************************************************************************
*********************************************************************************/

/*************************************************************************************
* ApplicationWindow definition
*
*
*
**************************************************************************************/

class ApplicationWindow: public QMainWindow
{
Q_OBJECT
public:
	ApplicationWindow(QString con);
	~ApplicationWindow();
	
	void InsertDebugText(QString debugmessage);
	void InsertTabmanText(QString debugmessage);
	void show();
	void SetSchemaName();
	void SetNewConnectString();
	
	QString username;
	QString password;
	QString database;
	QString schemaname;
	QString connectstring;
	bool connected;	

public slots:

	void Logon();
	void SetUnsetDebug();
	void SetUnsetTabmanMessages();
	
	void ShowDebugDialog();
	void ShowTabmanDialog();
	void ShowTableDialog();
	void ShowIndexDialog();
	void ShowConstraintDialog();
	void ShowColumnDialog();

	void HideDebugDialog();
	void HideTabmanDialog();
	void HideTableDialog();
	void HideIndexDialog();
	void HideConstraintDialog();
	void HideColumnDialog();
	
	void ShowHideDebugDialog();
	void ShowHideTabmanDialog();
	void ShowHideTableDialog();
	void ShowHideIndexDialog();
	void ShowHideConstraintDialog();
	void ShowHideColumnDialog();
	
	void ShowAbout();
	void ScanTabmanPipe();
	void StartSqlPlus();
	void DoFullRefresh();
	void ListClicked();
	void TreeClicked();
	void ShowObjectProperties();
	void SetNewObjectSelection(QString iname,QString itype,
							   QString iparentname,QString iparenttype);
	void NullSlot();
	
signals:
	void ConnectionChanged();
	void ListChanged();
	void TreeChanged();

private:
	QSplitter *split;
	SchemaTree *schematree;
	SchemaList *listview;
	SchemaTreeItem *root;
	
	DebugDialog *debugdialog;
	TabmanDialog *tabmandialog;
	TableDialog *tabledialog;
	
	ToolBar *toolbar;
	StatusBar *statusbar;
	MenuBar *menu;
	
	bool tabmanscanstate;
	bool debugstate;
	bool tabmanwindowstate;
	bool debugwindowstate;
	bool tablewindowstate;
	bool indexwindowstate;
	bool constraintwindowstate;
	bool columnwindowstate;
	
	QString itemname;
	QString itemtype;
	QString itemparentname;
	QString itemparenttype;
	
};


/*************************************************************************************
* DebugDialog definition
*
*
*
**************************************************************************************/

class DebugDialog: public QDialog
{
Q_OBJECT
public:
	DebugDialog(QWidget *parent,char *label);
	~DebugDialog();
	
	void InsertDebugText(QString debugmessage);
	void resizeEvent(QResizeEvent * r_event);
	
public slots:
	void hideEvent(QHideEvent *h);
	void ClearMessages();

signals:
	void Closed();

private:
	QPushButton *clear;
	QPushButton *hidebutton;
	ScrollText *debugmessages;
	QWidget *par;

};


/*************************************************************************************
* TabmanDialog definition
*
*
*
**************************************************************************************/

class TabmanDialog: public QDialog
{
Q_OBJECT
public:
	TabmanDialog(QWidget *parent,char *label);
	~TabmanDialog();
	
	void InsertTabmanText(QString tabmanmessage);
	void resizeEvent(QResizeEvent * r_event);
	void Blink();
	
public slots:
	void hideEvent(QHideEvent *h);
	void ClearMessages();
	void BlinkTimer();

signals:
	void Closed();

private:
	QPushButton *clear;
	QPushButton *hidebutton;
	ScrollText *tabmanmessages;
	QLabel *blinker;
	QWidget *par;
	QTimer *timer;

};

/*************************************************************************************
* TableDialog definition
*
*
*
**************************************************************************************/

class TableDialog: public QDialog
{
Q_OBJECT
public:
	TableDialog(QString name,QString owner,QWidget *parent,const char *name);
	~TableDialog();
	
	void SetTableName(QString name,QString owner);
	void GetTableStatistics();
	void resizeEvent(QResizeEvent * r_event);
	
public slots:
	void hideEvent(QHideEvent *h);
	void RefreshProperties();
	void UserMadeChanges(const QString&);
	void DoAlterTable();
	void AnalyzeTable();
	
signals:
	void Closed();
	
private:

	QGroupBox *properties;
	QGroupBox *statistics;
	
	QLineEdit *tablename ;
	QLineEdit *owner ;
	QLineEdit *new_tablename ;
	QLineEdit *new_owner ;
	QLineEdit *tablespace ;
	QLineEdit *pct_free ;
	QLineEdit *pct_used ;
	QLineEdit *pct_incr ;
	QLineEdit *next_extent ;
	QLineEdit *min_extents ;
	QLineEdit *max_extents ;
	QLineEdit *max_trans ;
	QLineEdit *min_trans ;
	QLineEdit *ini_trans ;
	QLineEdit *initial_extent ;
	QLineEdit *free_lists ;
	QLineEdit *free_list_groups ;
	
	QLineEdit *rowcount;
	QLineEdit *chained_rows;
	QLineEdit *avg_row_len;
	QLineEdit *blocks_on_freelist;
	QLineEdit *avg_free_space;
	QLineEdit *blocks_not_used;
	QLineEdit *blocks_used;
	QLineEdit *last_analyze;
	QLineEdit *sample_size;

	QPushButton *ok;
	QPushButton *revert;
	QPushButton *cancel;
	QPushButton *analyze;

};

/*************************************************************************************
* AboutDialog definition
*
*
*
**************************************************************************************/

class AboutDialog : public QDialog 
{
Q_OBJECT
public:
	AboutDialog( QWidget *parent, const char *name );
	~AboutDialog();
	
private:
	QPushButton *ok;
};

/*************************************************************************************
* LoginDialog definition
*
*
*
**************************************************************************************/

class LoginDialog : public QDialog 
{
Q_OBJECT
public:
	
	LoginDialog( QWidget *parent, const char *name );
	~LoginDialog();

	exec();

	QLineEdit *username, *passwd, *database;
};


/********************************************************************************
*********************************************************************************
* Other classes
*********************************************************************************
*********************************************************************************/


/*************************************************************************************
* SchemaTree definition
*
*
*
**************************************************************************************/

class SchemaTree: public QListView
{
Q_OBJECT
public:
	SchemaTree(QString schemaname,QWidget *parent,char *label,SchemaList *list);
	~SchemaTree();
	void resizeEvent(QResizeEvent *);
	
	/* pointer to the righthandside list */
	SchemaList *listview;

public slots:
	void SchemaTreeClick();
	void RefreshTree();

private:
	QWidget *par;

};


/*************************************************************************************
* SchemaTreeItem definition
*
*
*
**************************************************************************************/

class SchemaTreeItem: public QListViewItem
{

public:
	SchemaTreeItem( SchemaTree * parent ,QString label,QString type);
	SchemaTreeItem(SchemaTreeItem *parent,QString label,QString typeset,QString p1,QString p2,bool isfolder,bool isexpandable);
	~SchemaTreeItem();

	char *object_name,*object_type;
	
	void setOpen(bool);
	
	SchemaTreeItem *par;
	bool	folder;
	int	level;
	QString type;
	QString parm1;
	QString parm2;
		
};

/*************************************************************************************
* SchemaList definition
*
*
*
**************************************************************************************/

class SchemaList: public QListView
{
Q_OBJECT
public:
	SchemaList(QWidget *parent,char *label);
	~SchemaList();
	
	/* methods */
	int RefreshSchemaList();
	
	/* Setters */
	int SetNewProperties(QString new_name,
			     QString new_type,
			     QString new_parent_name,
			     QString new_parent_type,
			     int itemlevel,
			     bool isfolder,
			     QString new_param1,
			     QString new_param2);
	void focusInEvent ( QFocusEvent * e);
	
public slots:
	void SchemaListClick();
	void SchemaListDoubleClick();
	void SchemaListRightClick(QListViewItem *clickeditem, const QPoint &pos, int a);
	void SchemaListItemDrop();
	void DoRefresh();

signals:
	void ObjectCountChanged(int);
	void NewObjectSelected(QString iname,QString itype,
						   QString iparentname,QString iparenttype);
	void Click();

private:
	QWidget *par;
	
	/* properties */
	int level;
	bool folder;
	QString name;
	QString type;
	QString parent_name;
	QString parent_type;
	QString param1;
	QString param2;


};

/*************************************************************************************
* SchemaListItem definition
*
*
*
**************************************************************************************/

class SchemaListItem: public QListViewItem
{

public:
	SchemaListItem(SchemaList * parent,QString par_type,QString par_name,QString this_type,
			 QString this_name,QString  p1,QString p2,bool o, bool p,
			  QString label1,  QString label2 = 0,  QString label3 = 0,  QString label4 = 0,
			  QString label5 = 0,  QString label6 = 0,  QString label7 = 0,  QString label8 = 0 );
	~SchemaListItem();
	
	/*void setOpen(bool);
	void setup();*/
	
	QString parent_name;
	QString parent_type;
	QString type;
	QString name;
	QString parm1;
	QString parm2;
	bool	dclickable;
	bool	rclickable;
	
private:
	QListView *par;		
};


/*************************************************************************************
* ScrollText definition
*
*
*
**************************************************************************************/

class ScrollText: public QMultiLineEdit
{
Q_OBJECT
public:
	ScrollText(QWidget *parent,char *label);
	~ScrollText();

	void append(QString str);
	
private:
	QWidget *par;

};

/*************************************************************************************
* StatusBar definition
*
*
*
**************************************************************************************/

class StatusBar: public QStatusBar
{
Q_OBJECT
public:
	StatusBar(QWidget *parent,char *name);
	~StatusBar();

public slots:
	void ShowConnectString(QString connectstring);
	void ShowObjectCount(int count);
	
private:
	QWidget *par;
	QLabel* connectstring;
	QLabel* objectcount;

};

/*************************************************************************************
* MenuBar definition
*
*
*
**************************************************************************************/

class MenuBar: public QMenuBar
{
Q_OBJECT
public:
	MenuBar(QWidget *parent,char *name);
	~MenuBar();

	bool GetDebugState()
		{return this->view->isItemChecked(dbggenerate);};
	bool GetTabmanState()
		{return this->view->isItemChecked(tabmanscan);};
	bool GetDebugWindowState()
		{return this->view->isItemChecked(dbgwindow);};
	bool GetTabmanWindowState()
		{return this->view->isItemChecked(tabmanwindow);};
	bool GetTableWindowState()
		{return this->view->isItemChecked(tablewind);};
	bool GetIndexWindowState()
		{return this->view->isItemChecked(indexwind);};
	bool GetConstraintWindowState()
		{return this->view->isItemChecked(constraintwind);};
	bool GetColumnWindowState()
		{return this->view->isItemChecked(columnwind);};

public slots:
	void SetDebugState(bool state)
		{this->view->setItemChecked(dbggenerate,state);};
	void SetTabmanState(bool state)
		{this->view->setItemChecked(tabmanscan,state);};
	void SetDebugWindowState(bool state)
		{this->view->setItemChecked(dbgwindow,state);};
	void SetTabmanWindowState(bool state)
		{this->view->setItemChecked(tabmanwindow,state);};
	void SetTableWindowState(bool state)
		{this->view->setItemChecked(tablewind,state);};
	void SetIndexWindowState(bool state)
		{this->view->setItemChecked(indexwind,state);};
	void SetConstraintWindowState(bool state)
		{this->view->setItemChecked(constraintwind,state);};
	void SetColumnWindowState(bool state)
		{this->view->setItemChecked(columnwind,state);};

private:
	QWidget *par;
	
	QPopupMenu *file;
	QPopupMenu *view;
	QPopupMenu *tools;
	QPopupMenu *help;
	
	int dbgwindow;
	int dbggenerate;
	int tabmanwindow;
	int tabmanscan;
	int tablewind;
	int indexwind;
	int constraintwind;
	int columnwind;
};

/*************************************************************************************
* ToolBar definition
*
*
*
**************************************************************************************/

class ToolBar: public QToolBar
{
Q_OBJECT
public:
	ToolBar(ApplicationWindow *parent,char *name);
	~ToolBar();

	void SetTabmanButtonState(bool state)
		{this->tabmanwind->setOn(state);};
		
	void SetTableButtonState(bool state)
		{this->tablewind->setOn(state);};
		
	void SetIndexButtonState(bool state)
		{this->indexwind->setOn(state);};
		
	void SetConstrButtonState(bool state)
		{this->constrwind->setOn(state);};
		
	void SetColButtonState(bool state)
		{this->colwind->setOn(state);};

private:
	QWidget *par;
	
	QToolButton *quit;
	QToolButton *connect;
	QToolButton *refresh;
	QToolButton *drop;
	QToolButton *prop;
	QToolButton *tabmanwind;
	QToolButton *tablewind;
	QToolButton *indexwind;
	QToolButton *constrwind;
	QToolButton *colwind;
	
	
};

#endif