/***************************************************************************
                          schematree.cpp  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id$
 *
 */

#include "qtabman.h"

SchemaTree::SchemaTree(QString shemaname,QWidget *parent,char *label,SchemaList *list)
	:QListView(parent,label)
{
	par = parent;
	listview = list;
	
	this->setTreeStepSize( 15 );
	this->addColumn("Schema tree");
	this->setRootIsDecorated(TRUE);

	/* create the treeroot */
	SchemaTreeItem *root = new SchemaTreeItem(this,"Schema root","ROOT");
	root->setOpen(TRUE);
	
	/* disable sorting */
	this->setSorting(-1,FALSE);

	QObject::connect( this, SIGNAL(selectionChanged()),
			          this, SLOT(SchemaTreeClick()) );
}

SchemaTree::~SchemaTree()
{}
void SchemaTree::resizeEvent(QResizeEvent *e)
{
	this->setColumnWidth (0, this->width() );
	QListView::resizeEvent(e);
}

void SchemaTree::SchemaTreeClick()
{
	int tel,aantal,i;
	char **data1=NULL;
	char **data2=NULL;
	char **data3=NULL;
	char **data4=NULL;
	char *err=NULL;
	SchemaTreeItem *treeitem;
	
	/* get a pointer to the currently selected item */
	treeitem = (SchemaTreeItem *)this->selectedItem();
	
	/*Debug("--------Begin SchemaTreeClick------------------------------");
	Debug("%s was selected",(const char*)treeitem->text(0));
	Debug("treeitem properties are :");
	Debug("     folder = '%d'",(const char*)treeitem->folder);
	Debug("     level  = '%d'",(const char*)treeitem->level);
	Debug("     name   = '%s'",(const char*)treeitem->text(0));
	Debug("     type   = '%s'",(const char*)treeitem->type);
	Debug("     parm1  = '%s'",(const char*)treeitem->parm1);
	Debug("     parm2  = '%s'",(const char*)treeitem->parm2);
	Debug("--------End SchemaTreeClick------------------------------");*/
	
	
	
	if(treeitem) /* make sure there is an item selected */
	{
		switch (treeitem->level)
		{
		case 1: /*root select (first level select)*/
			/* Set the current properties for the SchemalistView */
			if(listview->SetNewProperties(QString("ROOT"),
				     	 	      QString("SCHEMA ROOT"),
				     	  	      NULL,
				     	  	      NULL,
				     	  	      treeitem->level,
				     	  	      treeitem->folder,
				     	  	      treeitem->parm1,
				     	  	      treeitem->parm2) )
				Debug("Set failed");
		
			break;
		case 2:
		case 3:/*second an third level select*/
			/* Set the current properties for the SchemalistView */
			if(listview->SetNewProperties(treeitem->text(0),
				     	  	      treeitem->type,
				     	  	      NULL,
				     	  	      NULL,
				     	  	      treeitem->level,
				     	  	      treeitem->folder,
				     	  	      treeitem->parm1,
				     	  	      treeitem->parm2) )
				Debug("Set failed");
		
			break;
		case 4: /*fourth level select*/
			/* Set the current properties for the SchemalistView */
			if(listview->SetNewProperties(treeitem->text(0),
				     	  	      treeitem->type,
				     	  	      treeitem->par->text(0),
				     	  	      treeitem->par->type,
				     	  	      treeitem->level,
				     	  	      treeitem->folder,
				     	  	      treeitem->parm1,
				     	  	      treeitem->parm2) )
				Debug("Set failed");
		
			break;
		}
	} /* end if(treeitem */
}

void SchemaTree::RefreshTree()
{
	Debug("Tree needs refresh");
	SchemaTreeItem *rootitem,*nextitem,*child;

	/* get the root */
	rootitem = (SchemaTreeItem*) this->firstChild();

	/* first item under the root */
	nextitem = (SchemaTreeItem*) rootitem->firstChild();
	while(nextitem)
	{
		Debug("Refresh current item = %s",(const char*)nextitem->text(0));
		if(nextitem->childCount())
		{
			Debug("--> %s has childs : so delete",(const char*)nextitem->text(0));
			child = (SchemaTreeItem*)nextitem->firstChild();
			
			/* delete all childs */
			while( child )
			{
				delete child;
				child = (SchemaTreeItem*)child->nextSibling();
			}

			/* reopen the item to refresh */
			if(nextitem->isOpen())
			{
				Debug("--> %s was open : so reopen",(const char*)nextitem->text(0));
				nextitem->setOpen(TRUE);
			}
		}
		if (!nextitem->isExpandable())
		{
			nextitem->setExpandable( TRUE );
			nextitem->setOpen( FALSE );
		}
		nextitem=(SchemaTreeItem*)nextitem->nextSibling();
	}
}
