/***************************************************************************
                          schematreeitem.cpp  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : j.struyve@village.uunet.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id$
 *
 */

#include "qtabman.h"
#include <stdio.h>

SchemaTreeItem::SchemaTreeItem(SchemaTree *parent,QString label,QString typeset)
	:QListViewItem((QListView*)parent,label)
{
	/* this is a constructor for the root object *:
	/* no parent is needed, root is the to level in the tree (level 1) */
	
	par 	= NULL;
	folder 	= TRUE;
	level 	= 1;
	type 	= typeset;
}

SchemaTreeItem::SchemaTreeItem(SchemaTreeItem *parent,QString label,QString typeset,
				QString p1,QString p2,bool isfolder,bool isexpandable)
	:QListViewItem(parent,label)
{

	par = parent;
	
	/* increment the level */
	level= parent->level+1;
	
	/* set options */
	folder = isfolder;
	this->setExpandable(isexpandable);
	
	/* copy parameters */
	type = typeset;
	if (p1)	parm1= strdup(p1);
	if (p2)	parm2= strdup(p2);
	
	/* set the correct icon next to the item */
	if(!strcmp(this->type,"TABLE")||!strcmp(this->type,"TABLES"))
	{
		QPixmap *icon = new QPixmap((const char **)table_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"PACKAGE")||!strcmp(this->type,"PACKAGES"))
	{
		QPixmap *icon = new QPixmap((const char **)package_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"PROCEDURE")||!strcmp(this->type,"PROCEDURES"))
	{
		QPixmap *icon = new QPixmap((const char **)procedure_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"FUNCTION")||!strcmp(this->type,"FUNCTION"))
	{
		QPixmap *icon = new QPixmap((const char **)function_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"PACKAGE BODY"))
	{
		QPixmap *icon = new QPixmap((const char **)packagebody_xpm);
		this->setPixmap(0,*icon);
		this->setExpandable(FALSE);
	}
	else if(!strcmp(this->type,"INDEX")||!strcmp(this->type,"INDEXES"))
	{
		QPixmap *icon = new QPixmap((const char **)index_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"VIEW")||!strcmp(this->type,"VIEW"))
	{
		QPixmap *icon = new QPixmap((const char **)view_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"CONSTRAINT")||!strcmp(this->type,"CONSTRAINT"))
	{
		QPixmap *icon = new QPixmap((const char **)constraint_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"TRIGGER")||!strcmp(this->type,"TRIGGERS"))
	{
		QPixmap *icon = new QPixmap((const char **)trigger_xpm);
		this->setPixmap(0,*icon);
	}
	else if(!strcmp(this->type,"COLUMN")||!strcmp(this->type,"COLUMNS"))
	{
		QPixmap *icon = new QPixmap((const char **)column_xpm);
		this->setPixmap(0,*icon);
	}
}

SchemaTreeItem::~SchemaTreeItem()
{}

void SchemaTreeItem::setOpen(bool o)
{
	/* oracle function parameters */
	int aantal,tel;
	char **data1=NULL;
	char **data2=NULL;
	char **data3=NULL;
	char *err=NULL;
	bool p;
	
	
	switch (this->level)
	{
	case 1: /*root level unfold (first level)*/
		QApplication::setOverrideCursor( waitCursor );
		
		/* insert the object types under the root dir */
		if (!childCount())
		{
			new SchemaTreeItem(this,QString("Procedures") ,QString("PROCEDURE") ,NULL,NULL,TRUE,TRUE);
			new SchemaTreeItem(this,QString("Functions")  ,QString("FUNCTION")  ,NULL,NULL,TRUE,TRUE);
			new SchemaTreeItem(this,QString("Packages")   ,QString("PACKAGE")   ,NULL,NULL,TRUE,TRUE);
			new SchemaTreeItem(this,QString("Views")      ,QString("VIEW")      ,NULL,NULL,TRUE,TRUE);
			new SchemaTreeItem(this,QString("Indexes")    ,QString("INDEX")     ,NULL,NULL,TRUE,TRUE);
			new SchemaTreeItem(this,QString("Tables")     ,QString("TABLE")     ,NULL,NULL,TRUE,TRUE);
		}
		QApplication::restoreOverrideCursor();
		break;
		
	case 2: /*second level unfold (get the objects of the selected type)*/
		if( !childCount() && this->isExpandable())
		{
			QApplication::setOverrideCursor( waitCursor );
			
			/* determine if the item should be expandable */
			if(!strcmp(this->type,"INDEX")||!strcmp(this->type,"PROCEDURE")||!strcmp(this->type,"VIEW")) 
				p = FALSE;
			else
				p = TRUE;
			
			/* get the objects of the type that was unfolded */
			if(GetUserObjectsName(&data1,(const char*)this->type,&aantal,&err))
			{
				for(tel=0;tel<aantal;tel++)
				{
					new SchemaTreeItem(this,QString(data1[tel]),this->type,NULL,NULL,p,p);
				}
				if(aantal==0) this->setExpandable( FALSE );
				QApplication::restoreOverrideCursor();
			}
			else
			{
				QApplication::restoreOverrideCursor();
				QMessageBox::message("Error",err);
			}
		}
		break;
	case 3: /*third level unfold (build the object subtree)*/
		if( !childCount() && this->isExpandable())
		{
			if( !strcmp(this->type,"TABLE" ))
			{
				/* create the default sub-items */
				new SchemaTreeItem(this,QString("Triggers")    ,QString("TRIGGER")    ,NULL,NULL,TRUE,FALSE);
				new SchemaTreeItem(this,QString("Constraints") ,QString("CONSTRAINT") ,NULL,NULL,TRUE,FALSE);
				new SchemaTreeItem(this,QString("Indexes")     ,QString("INDEX")      ,NULL,NULL,TRUE,FALSE);
				new SchemaTreeItem(this,QString("Columns")     ,QString("COLUMN")     ,NULL,NULL,TRUE,FALSE);
			}
			
			if( !strcmp(this->type,"PACKAGE" ))
			{
				QApplication::setOverrideCursor( waitCursor );
				if(GetPackageBodyNames(&data1,&data2,&data3,(const char*)this->text(0),&aantal,&err))
				{
					for(tel=0;tel<aantal;tel++)
					{
						new SchemaTreeItem(this,QString(data3[tel]),
								QString("PACKAGE BODY"),
								QString(data1[tel]),
								QString(data2[tel])
								,FALSE,FALSE);
					}
					if(aantal==0) this->setExpandable( FALSE );
					QApplication::restoreOverrideCursor();
				}
				else
				{
					QApplication::restoreOverrideCursor();
					QMessageBox::message("Error",err);
				}
			}
		}
		break;
	}
	QApplication::restoreOverrideCursor();
	free(err);
	free(data1);
	free(data2);
	free(data3);
	QListViewItem::setOpen( o );
}
